/**
	Archivo: wtiff.c
**/

/** Escribe un archivo TIFF a partir de un MATRIX **/

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include "tiff.h"

unsigned char masktable[8] = {  0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01 };
 

/** Pone en un archivo una palabra ( 2 bytes ) en formato Intel **/
int fputword( FILE *fp, int n)
{
	fputc(n,fp);
	fputc((n >> 8),fp);
	return(ferror(fp));
}

/** Pone en archivo un entero largo ( 4 bytes ) en formato intel **/
int fputlong( FILE *fp, long n)
{
	fputc((char)n,fp);
	fputc((char)(n >> 8),fp);
	fputc((char)(n >> 16),fp);
	fputc((char)(n >> 24),fp);
	return(ferror(fp));
}


/** Escribe un campo TIFF en el  IFD **/
int writetiftag( FILE *fp, int tag, int type, long length, long offset)
{
	fputword(fp,tag);
	fputword(fp,type);
	fputlong(fp,length);
	fputlong(fp,offset);

	return(ferror(fp));
}


int get_line( MATRIX *MAT, char *ptr, int bytes, int n )
{
	int i;
	unsigned char * ptrIN;

	switch( MAT->type ){
		case 0:
		case 1:
			memset( ptr, 0, bytes );
               		ptrIN = ( unsigned char *  ) MAT->ptr[n];
               		for ( i=0; i< MAT->cols; i++ ) {
                       	   if ( *ptrIN++ > 0 )
                               	ptr[ i>>3 ] |= masktable[ i & 0x0007 ];
               		}
			break;
		case 2:
			ptrIN = ( unsigned char *  ) MAT->ptr[n];
			for( i=0; i<MAT->cols; ) {
				*ptr = (*ptrIN<<4) & 0xf0;
				ptrIN++;
				*ptr |= *ptrIN & 0x0f;
				ptrIN++;
				i += 2;
			}
			break;
		case 5:
			memcpy( ptr, MAT->ptr[n], bytes );
			break;
	}
}

/* Escribe un archivo TIFF  **/
/** Solo escribe imagenes en blanco y negro o en tonos de gris (16 o 256 ) **/ 
int write_tiff ( MATRIX * IN, char *name )
{
	FILE *fp;
	char *p,*pr;
	long pos;
	unsigned int width, depth;
	int i, bytes;
	long  _stripbytecounts;

	if( (fp=fopen( name, "w") )==NULL ) return (1);
	/* Calcula al tamanio de una linea */
	/** Modificacion  16/07/93 **/
	width = IN -> cols;
	depth = IN -> rows;
	switch ( IN->type ) {
		case 0:   /* Blanco y negro */
		case 1:
			bytes=pixels2bytes(width);
			break;
		case 2:   /* 16 tonos de gris */
			if ( width%2 ) bytes = (width >> 1) + 1;
			else bytes = width >> 1;
			break;
		case 5:   /* 256 tonos de gris */
			bytes = width;
			break;
		default: return(1);
	}
	_stripbytecounts = (long)bytes*(long)IN->rows;
	/* Localiza un buffer de una linea */
	if((p=(char *)malloc( bytes )) == NULL) return(MEMORY_ERROR);

	/* Escribe el encabezado */
	fputword(fp,II);
	fputword(fp,42);
	fputlong(fp,0L);

	/** Escribe el mapa de bits **/
	for(i=0;i<depth;++i) {
		get_line(IN, p, bytes, i);
		fwrite(p,1,bytes, fp);
	}

	pos=ftell(fp);
	/** Escribe los campos segun el tipo del archivo TGC **/
	switch( IN->type ) {
		case 0:
			fputword(fp,9);
			writetiftag(fp,SubfileType,TIFFshort,1L,1L);
			writetiftag(fp,ImageWidth,TIFFshort,1L,(long)width);
			writetiftag(fp,ImageLength,TIFFshort,1L,(long)depth);
                        writetiftag(fp,BitsPerSample,TIFFshort,1L,1L);
			writetiftag(fp,Compression,TIFFshort,1L, 1L);
			writetiftag(fp,PhotometricInterp,TIFFshort,1L,0L);
                        writetiftag(fp,StripOffsets,TIFFlong,1L,8L);
                        writetiftag(fp,RowsPerStrip,TIFFlong,1L,(long)depth);
       			writetiftag(fp,StripByteCounts,TIFFlong,1L,_stripbytecounts);
			break;
		case 1:
			fputword(fp,9);
			writetiftag(fp,SubfileType,TIFFshort,1L,1L);
			writetiftag(fp,ImageWidth,TIFFshort,1L,(long)width);
			writetiftag(fp,ImageLength,TIFFshort,1L,(long)depth);
			writetiftag(fp,BitsPerSample,TIFFshort,1L,1L);
			writetiftag(fp,Compression,TIFFshort,1L, 1L);
			writetiftag(fp,PhotometricInterp,TIFFshort,1L,1L);
			writetiftag(fp,StripOffsets,TIFFlong,1L,8L);
                        writetiftag(fp,RowsPerStrip,TIFFlong,1L,(long)depth);
       			writetiftag(fp,StripByteCounts,TIFFlong,1L,_stripbytecounts);
			break;
		case 2:
			fputword(fp,9);
			writetiftag(fp,SubfileType,TIFFshort,1L,1L);
			writetiftag(fp,ImageWidth,TIFFshort,1L,(long)width);
			writetiftag(fp,ImageLength,TIFFshort,1L,(long)depth);
			writetiftag(fp,BitsPerSample,TIFFshort,1L,4L);
			writetiftag(fp,Compression,TIFFshort,1L, 1L);
			writetiftag(fp,PhotometricInterp,TIFFshort,1L,1L);
			writetiftag(fp,StripOffsets,TIFFlong,1L,8L);
                        writetiftag(fp,RowsPerStrip,TIFFlong,1L,(long)depth);
       			writetiftag(fp,StripByteCounts,TIFFlong,1L,_stripbytecounts);
			break;
		case 5:
			fputword(fp,9);
			writetiftag(fp,SubfileType,TIFFshort,1L,1L);
			writetiftag(fp,ImageWidth,TIFFshort,1L,(long)width);
			writetiftag(fp,ImageLength,TIFFshort,1L,(long)depth);
			writetiftag(fp,BitsPerSample,TIFFshort,1L,8L);
			writetiftag(fp,Compression,TIFFshort,1L, 1L);
			writetiftag(fp,PhotometricInterp,TIFFshort,1L,1L);
			writetiftag(fp,StripOffsets,TIFFlong,1L,8L);
                        writetiftag(fp,RowsPerStrip,TIFFlong,1L,(long)depth);
       			writetiftag(fp,StripByteCounts,TIFFlong,1L,_stripbytecounts);
			break;
	}


	fputlong(fp,0L);
	fseek(fp,4L,SEEK_SET);
	fputlong(fp,pos);

	free(p);

	if(ferror(fp)){
		fclose( fp );
		return(BAD_FILE);
	}
	else{
		fclose(fp);
		return(GOOD_WRITE);
	}
}



